@@ -2,7 +2,7 @@
 # Package       HiPi::BCM2835::I2C
 # Description:  I2C Connection
 # Created       Wed Mar 13 13:40:32 2013
-# SVN Id        $Id: I2C.pm 76 2016-03-29 12:36:41Z Mark Dootson $
+# SVN Id        $Id: I2C.pm 84 2016-04-03 02:08:01Z Mark Dootson $
 # Copyright:    Copyright (c) 2013 Mark Dootson
 # Licence:      This work is free software; you can redistribute it and/or modify it 
 #               under the terms of the GNU General Public License as published by the 
@@ -18,11 +18,11 @@ use strict;
 use warnings;
 use parent qw( HiPi::Class );
 use HiPi::BCM2835 qw( :registers :i2c :clock );
-use HiPi::Constant qw( :raspberry :i2c );
+use HiPi::Constant qw( :raspberry :i2c :i2c_readmode);
 use Carp;
 
 __PACKAGE__->create_accessors( qw(
-    _hipi_baseaddr peripheral address _function_mode _clock_divider _baud_reference
+    _hipi_baseaddr peripheral address _function_mode _clock_divider _baud_reference readmode
 ));
 
 our $VERSION = '0.27';
@@ -62,9 +62,8 @@ use constant {
     $EXPORT_TAGS{i2c} = \@const;
 }
 
-
 sub set_baudrate {
-    my ($objorclass, $channel, $newval) = @_;
+    my ($objorclass, $newval ) = @_;
     
     $newval ||= 100000;
     $newval = 3816 if $newval < 3816;
@@ -82,17 +81,12 @@ sub set_baudrate {
     # make sure library is initialised
     HiPi::BCM2835::bcm2835_init();
     
-    # force some default values / ranges
-    unless( defined($channel) && ( ( $channel == BB_I2C_PERI_0  ) || ( $channel == BB_I2C_PERI_1 ) ) ){
-        croak('channel must be defined as constant BB_I2C_PERI_0 or BB_I2C_PERI_1');
-    }
-    
+    my $channel = ( RPI_BOARD_REVISION == 1 ) ? BB_I2C_PERI_0 : BB_I2C_PERI_1;
+        
     my $baseaddress = ( $channel == BB_I2C_PERI_0 )
         ? BCM2835_BSC0_BASE
         : BCM2835_BSC1_BASE;
-    
-    
-    
+
     # base library is hard coded for channel 1
     if($channel == BB_I2C_PERI_1 ) {
         HiPi::BCM2835::bcm2835_i2c_set_baudrate($newval);
@@ -104,13 +98,15 @@ sub set_baudrate {
 }
 
 sub get_baudrate {
-    my ($objorclass, $channel) = @_;
+    my ($objorclass) = @_;
     
     # As instance method return our own baudrate
     if(ref($objorclass)) {
         return $objorclass->_baud_reference;
     }
     
+    my $channel = ( RPI_BOARD_REVISION == 1 ) ? BB_I2C_PERI_0 : BB_I2C_PERI_1;
+    
     my $cdiv = _get_current_clockdivider($channel);
     return _get_baudrate_from_clockdivider($cdiv);
 }
@@ -147,6 +143,7 @@ sub new {
         address      => 0,
         peripheral   => ( RPI_BOARD_REVISION == 1 ) ? BB_I2C_PERI_0 : BB_I2C_PERI_1,
         _function_mode => 'hipi',
+        readmode => I2C_READMODE_SYSTEM,
     );
     
     # get user params
@@ -295,7 +292,7 @@ sub delay {
 
 sub delayMicroseconds {
     my($class, $micros) = @_;
-    HiPi::BCM2835::bcm2835_delayMicroSeconds( $micros );
+    HiPi::BCM2835::bcm2835_delayMicroseconds( $micros );
 }
 
 #-------------------------------------
@@ -308,7 +305,14 @@ sub delayMicroseconds {
 
 sub bus_write { shift->i2c_write( @_ ); }
 
-sub bus_read { shift->i2c_read_register( @_ ); }
+sub bus_read {
+    my( $self, @params ) = @_;
+    my $readfunc = 'i2c_read_register';
+    if($self->readmode == I2C_READMODE_REPEATED_START  ) {
+        $readfunc = 'i2c_read_register_rs';
+    }
+    $self->$readfunc(@params);
+}
 
 sub bus_read_bits {
     my($self, $regaddr, $numbytes) = @_;
